package com.cdrussell.numbergames;

import java.util.Locale;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.speech.tts.TextToSpeech;
import android.speech.tts.TextToSpeech.OnInitListener;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.EditorInfo;
import android.widget.TextView;
import android.widget.TextView.OnEditorActionListener;
import android.widget.Toast;

/**
 * Counting up sequentially game.
 * @author Russell
 *
 */
public class CountUpActivity extends Activity implements OnInitListener
{
	private static final String TAG = "CountUpActivit";
	private PermanentKeyboardEditText outputEditText;
	private TextView currentNumberText;
	private int currentNumber = 0;
	private TextToSpeech myTTS;
	

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_count_up);
		
		getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);

		currentNumberText = (TextView)findViewById(R.id.currentNumberTextView);
		outputEditText = (PermanentKeyboardEditText) findViewById(R.id.outputTextView);
		//outputEditText.setImeActionLabel("Enter", EditorInfo.IME_ACTION_DONE);
		outputEditText.setOnEditorActionListener(new OnEditorActionListener()
		{
			@Override
			public boolean onEditorAction(TextView v, int actionId, KeyEvent event)
			{
				if(actionId == EditorInfo.IME_ACTION_DONE)
				{
					Log.d(TAG, "IME ACTION DONE");
				
					String text = outputEditText.getText().toString();
					try
					{
						
						if(text.length() == 0)
							return true;
						
						Log.d(TAG,  "Got text: " + text);
						
						int number = Integer.parseInt(text);
						
						Log.d(TAG, "Parsed to integer successfully: " + number);
						
						numberEntered(number);
					}
					catch(NumberFormatException e)
					{
						Log.d(TAG, "Not a number: " + text, e);
					}
				}
				return true;
			}
		});
		
		Intent tts = new Intent();
		tts.setAction(TextToSpeech.Engine.ACTION_CHECK_TTS_DATA);
		startActivityForResult(tts, 0);
	}
	
	public void playAnotherGameButtonPressed(View v)
	{
		finish();
	}
	
	private void numberEntered(int numberEntered)
	{
		outputEditText.setText("");
		if (numberEntered == currentNumber + 1)
		{
			
			Log.i(TAG, String.format("%d is correct", numberEntered));
			
			// right answer
			currentNumberText.setText(String.valueOf(++currentNumber));
			
			speak(String.valueOf(currentNumber));
		}
		else
		{
			speak("Try again. What comes after " + currentNumber + "?");
			
			Log.i(TAG, String.format("%d is incorrect (was looking for %s)", numberEntered, numberEntered + 1));
			
			// wrong answer
			Toast.makeText(this, String.format("%d is wrong", numberEntered), Toast.LENGTH_SHORT).show();
		}
	}
	
	private void speak(String speech)
	{
		if(myTTS != null)
			myTTS.speak(speech, TextToSpeech.QUEUE_FLUSH, null);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		super.onActivityResult(requestCode, resultCode, data);
		
		if(requestCode == 0)
		{
			if(resultCode == TextToSpeech.Engine.CHECK_VOICE_DATA_PASS)
			{
				myTTS = new TextToSpeech(this, this);
			}
			else
			{
				Intent installTTS = new Intent();
				installTTS.setAction(TextToSpeech.Engine.ACTION_INSTALL_TTS_DATA);
				startActivity(installTTS);
			}
		}
	}

	@Override
	protected void onResume()
	{
		super.onResume();
		outputEditText.requestFocus();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.count_up, menu);
		return true;
	}
	
	@Override
	protected void onDestroy()
	{
		super.onDestroy();
		if(myTTS != null)
			myTTS.shutdown();
	}

	@Override
	public void onInit(int initStatus)
	{
		if(initStatus == TextToSpeech.SUCCESS)
		{
			myTTS.setLanguage(Locale.getDefault());
		}
	}

}
