package com.cdrussell.numbergames;

import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Random;

import se.fnord.android.layout.PredicateLayout;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.speech.tts.TextToSpeech;
import android.speech.tts.TextToSpeech.OnInitListener;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.View;
import android.view.View.OnFocusChangeListener;
import android.view.inputmethod.EditorInfo;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.TextView.OnEditorActionListener;

public class MissingNumberActivity extends Activity implements OnInitListener
{
	private static final String TAG = "MainMenuActivity";

	private static final String EXCLUDED_NUMBERS_KEY = "saved_instance_excluded_numbers_key";

	private int minNumber;
	private int maxNumber;
	private Map<String, Boolean> answers;

	private TextToSpeech tts;
	private int correct;

	private ArrayList<Integer> excludedNumbers;
	private TextView scoreTextView;

	private Random random;

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_missing_number);

		random = new Random();

		answers = new HashMap<String, Boolean>();

		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
		//minNumber = prefs.getInt("pref_key_missing_number_min_value", 0);
		//maxNumber = prefs.getInt("pref_key_missing_number_max_value", 20);

		minNumber = 0;
		maxNumber = 20;
		
		if (savedInstanceState == null)
		{
			calcExcludedNumbers();
		}
		else
		{
			excludedNumbers = savedInstanceState.getIntegerArrayList(EXCLUDED_NUMBERS_KEY);

			Log.d(TAG, "Excluding numbers: " + excludedNumbers);
		}

		PredicateLayout ll = (PredicateLayout) findViewById(R.id.missing_number_text_view_container);
		for (int i = minNumber; i < maxNumber; i++)
		{
			LayoutInflater inflater = LayoutInflater.from(this);
			if (excludedNumbers.contains(i))
			{
				final EditText editView = (EditText) inflater.inflate(R.layout.missing_number_edit_view, null);
				editView.setOnFocusChangeListener(new OnFocusChangeListener()
				{
					@Override
					public void onFocusChange(View v, boolean hasFocus)
					{
						if (hasFocus)
						{
						}
						else
						{
							validateNumber(editView);
						}
					}

				});
				editView.setOnEditorActionListener(new OnEditorActionListener()
				{
					@Override
					public boolean onEditorAction(TextView v, int actionId, KeyEvent event)
					{
						boolean valid = true;

						if (actionId == EditorInfo.IME_ACTION_DONE)
						{
							Log.d(TAG, "IME ACTION DONE");
							valid = validateNumber(editView);
							if (correct == excludedNumbers.size())
							{
								speak(v.getText().toString() + ". Well done, you got all " + excludedNumbers.size()
										+ " missing numbers!", false);

								Builder ad = new AlertDialog.Builder(MissingNumberActivity.this);
								ad.setMessage("You got them all right!");
								ad.setTitle("Well Done");
								ad.setNeutralButton("OK", new OnClickListener()
								{
									@Override
									public void onClick(DialogInterface dialog, int which)
									{
										finish();
									}
								});
								ad.show();
							}
							else
							{
								if (valid)
									speak(v.getText().toString());
							}
						}
						else if (actionId == EditorInfo.IME_ACTION_NEXT)
						{
							Log.d(TAG, "IME ACTION NEXT");
							valid = validateNumber(editView);
							if (valid)
								speak(editView.getText().toString());
						}
						if (!valid)
						{
							return true;
						}

						return false;
					}
				});

				editView.setId(1000 + i);
				ll.addView(editView);
			}
			else
			{
				TextView v = (TextView) inflater.inflate(R.layout.number_text_view, null);
				v.setText(String.valueOf(i));
				v.setId(1000 + i);
				ll.addView(v);
			}
		}

		if (savedInstanceState != null)
		{
			for (int i = 0; i < maxNumber; i++)
			{
				View v = findViewById(1000 + i);
				if (v instanceof TextView)
				{
					TextView tv = (TextView) v;

					tv.setText(savedInstanceState.getString(String.valueOf(1000 + i)));
					if (tv.getText().equals(String.valueOf(1000 + i)))
					{
						answers.put(String.valueOf(1000 + i), true);
					}
					else
					{
						answers.put(String.valueOf(1000 + i), false);
					}
				}
			}
		}

		// scoreTextView = (TextView)findViewById(R.id.score_text_view);
		updateScore();

		Intent tts = new Intent();
		tts.setAction(TextToSpeech.Engine.ACTION_CHECK_TTS_DATA);
		startActivityForResult(tts, 0);
	}

	private void calcExcludedNumbers()
	{
		excludedNumbers = new ArrayList<Integer>();

		for (int i = 0; i < 5; i++)
		{
			Integer number = random.nextInt(20);
			if (excludedNumbers.contains(number))
			{
				i--;
				continue;
			}

			excludedNumbers.add(number);
		}
	}

	private void updateScore()
	{
		correct = 0;
		for (Boolean answer : answers.values())
		{
			if (answer)
				correct++;
		}

		double percent = (correct / excludedNumbers.size());
		DecimalFormat df = new DecimalFormat("0");
		df.setRoundingMode(RoundingMode.DOWN);
		String percentString = df.format(percent);

		String output = String.format("%d more to get right\nYou are %s%% complete", excludedNumbers.size() - correct,
			percentString);

		// scoreTextView.setText(output);
		Log.d(TAG, output);

	}

	private boolean validateNumber(final EditText editView)
	{
		String input = editView.getText().toString();
		if (input.length() > 0)
		{
			String number = String.valueOf(editView.getId() - 1000);
			if (number.equals(input))
			{
				editView.setTextColor(getResources().getColor(android.R.color.holo_green_dark));
				answers.put(String.valueOf(editView.getId() - 1000), true);
				updateScore();
				return true;
			}
			else
			{
				int prev = (Integer.valueOf(number) - 1);
				if (prev == -1)
				{
					speak("No, which number comes first?");
				}
				else
				{
					speak("No, what comes after " + prev + "?");
				}
				answers.put(String.valueOf(editView.getId() - 1000), false);
				editView.setTextColor(getResources().getColor(android.R.color.holo_red_dark));
				updateScore();
				return false;
			}
		}
		updateScore();
		return false;
	}

	@Override
	protected void onResume()
	{
		super.onResume();
	}

	private void speak(String speech)
	{
		speak(speech, true);
	}

	private void speak(String speech, boolean interrupt)
	{
		Log.d(TAG, "Trying to speak: " + speech);

		if (tts != null)
		{
			if (interrupt)
				tts.speak(speech, TextToSpeech.QUEUE_FLUSH, null);
			else
				tts.speak(speech, TextToSpeech.QUEUE_ADD, null);
		}
		else
		{
			Log.wtf(TAG, "TTS is null!");
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		super.onActivityResult(requestCode, resultCode, data);

		if (requestCode == 0)
		{
			if (resultCode == TextToSpeech.Engine.CHECK_VOICE_DATA_PASS)
			{
				tts = new TextToSpeech(this, this);
			}
			else
			{
				Intent installTTS = new Intent();
				installTTS.setAction(TextToSpeech.Engine.ACTION_INSTALL_TTS_DATA);
				startActivity(installTTS);
			}
		}
	}

	@Override
	protected void onSaveInstanceState(Bundle outState)
	{
		super.onSaveInstanceState(outState);

		outState.putIntegerArrayList(EXCLUDED_NUMBERS_KEY, excludedNumbers);

		for (int i = 0; i < maxNumber; i++)
		{
			View v = findViewById(1000 + i);
			String value = null;
			if (v instanceof TextView)
			{
				value = ((TextView) v).getText().toString();

			}

			if (value != null)
				outState.putString(String.valueOf(1000 + i), value);
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.missing_number, menu);
		return true;
	}

	@Override
	public void onInit(int initStatus)
	{
		if (initStatus == TextToSpeech.SUCCESS)
		{
			tts.setLanguage(Locale.getDefault());
		}
	}

}
